from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.log import logger
from nonebot.params import CommandArg

from .data_source import get_recommend_tags

recruit = on_command("公开招募", aliases={"公招", "方舟公招", "公招tag"}, priority=5, block=True)


@recruit.handle()
async def _(arg: Message = CommandArg()):
    logger.info(f"{arg}")
    taglist = arg.extract_plain_text().strip().split()
    rst = await get_recommend_tags(taglist)

    if rst is None:
        rst = "获取公招推荐数据失败！请检查网络连接并稍后重试"

    if not rst:
        rst = "当前没有必出稀有干员的公招标签"

    await recruit.finish(rst, at_sender=True)


"""如果无法截图"""
import time

WEEKDAYS = {
    "Mon": 1,
    "Tue": 2,
    "Wed": 3,
    "Thu": 4,
    "Fri": 5,
    "Sat": 6,
    "Sun": 7
}

SOURCE = {
    "作战记录(LS)": [1, 2, 3, 4, 5, 6, 7],
    "龙门币(CE)": [2, 4, 6, 7],
    "采购凭证(AP)": [1, 4, 6, 7],
    "碳素(SK)": [1, 3, 5, 6],
    "技巧概要(CA)": [2, 3, 5, 7],
    "医疗、重装芯片(PR-A)": [1, 4, 5, 7],
    "术师、狙击芯片(PR-B)": [1, 2, 5, 6],
    "辅助、先锋芯片(PR-C)": [3, 4, 6, 7],
    "特种、近卫芯片(PR-D)": [2, 3, 6, 7]
}


async def alter_plan():
    weekday = WEEKDAYS[time.strftime("%a", time.localtime())]
    return [name for name, day in SOURCE.items() if weekday in day]


"""网络请求相关异常"""
from . import ArkBaseException


class NoMethodException(ArkBaseException):
    """请求方式不是 GET 或 POST"""
    def __init__(self, msg: str = "请求方式只能为 GET 或 POST ", **kwargs):
        super().__init__(msg, **kwargs)
        self.msg = msg



from typing import Union, List


def merge_msg(msg_list: List[str], uin: Union[int, str]) -> List[dict]:
    """
    生成自定义合并消息
    :param msg_list: 消息列表
    :param uin: 发送者 QQ
    """
    uin = int(uin)
    mes_list = []
    for _message in msg_list:
        data = {
            "type": "node",
            "data": {
                "name": "test",
                "uin": f"{uin}",
                "content": _message,
            },
        }
        mes_list.append(data)
    return mes_list



"""塞壬唱片, https://monster-siren.hypergryph.com/"""
from lxml import etree
from nonebot.log import logger

from ..._exceptions import APICodeException
from ..._utils import get_api, request_

CATEGORY = {
    1: "新歌发布",
    5: "上线致辞",
    7: "资讯速递",
    8: "艺人近况",
    11: "特别电台"
}

APIS = get_api("monster-siren")


async def get_album_news() -> list:
    """获取最新三条公告"""
    url = APIS["news"]["url"]
    method = APIS["news"]["method"]
    response = await request_(url=url, method=method)

    data = response.json()
    if data['code'] != 0:
        logger.error(f"塞壬唱片最新专辑获取失败")
        raise APICodeException(status=response['code'])

    data = response.json()['data']
    logger.info(f"album_data: {data}")
    result = []
    for _ in data['list'][:3]:
        result.append(await process_album_news(_))
    logger.info(f"album_result: {result}")
    return result


async def process_album_news(news_data: dict) -> dict:
    """获取公告中的图片等信息"""
    cid = news_data['cid']
    url = APIS["news-data"]["url"].format(cid)
    method = APIS["news-data"]["method"]
    response = await request_(url=url, method=method)

    data = response.json()
    if data['code'] != 0:
        raise APICodeException(status=response['code'])

    data = response.json()['data']

    title = data['title']
    author = data['author']
    date = data['date']

    cate = data['cate']
    cate = CATEGORY[cate] if cate in CATEGORY.keys() else None

    content = data['content']
    dom = etree.HTML(content, etree.HTMLParser())
    img = dom.xpath("//img/@src")
    img = img[0] if img else None

    return {
        "title": title,
        "date": date,
        "cate": cate,
        "author": author,
        "img": img,
        "url": f"https://monster-siren.hypergryph.com/info/{cid}"
    }







"""获取干员信息"""
from nonebot import on_command
from nonebot.exception import ActionFailed
from nonebot.params import CommandArg
from nonebot.adapters.onebot.v11 import Message, MessageSegment

from .data_source import *

query_operator = on_command("干员", aliases={"方舟干员", "明日方舟干员"})


@query_operator.handle()
async def _(arg: Message = CommandArg()):
    try:
        name = arg.extract_plain_text().strip()
        op = OperatorInfo(name)
        build = BuildOperatorImage(op)
        img = build.build_whole_image()

        img = MessageSegment.image(img)
        await query_operator.finish(Message(img))
    except ActionFailed as e:
        await query_operator.finish("图片发送失败！", at_sender=True)


import os
import json
from .._exceptions import *


def get_api(field: str):
    """
    获取 API。
    :param field: API 所属分类，即 _apis 下的文件名（不含后缀名）
    :return dict, 该 API 的内容。
    """
    path = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "_apis", f"{field.lower()}.json"))
    print(path)
    if os.path.exists(path):
        with open(path, encoding="utf8") as f:
            return json.loads(f.read())
    else:
        raise APINonExistenceException


"""异常基类"""
from typing import Union


class ArkBaseException(Exception):
    """基础异常类"""
    def __init__(self, msg: str = "出现了错误，但是未说明具体原因。", details: Union[str, int] = ""):
        super().__init__(msg)
        self.msg = f"{msg} - {details}"

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return self.msg



import contextlib
import os
from typing import Union

from PIL import Image, ImageFont
from PIL.ImageDraw import Draw, ImageDraw
from nonebot import get_driver
import json
from pathlib import Path
from .config import Config


operator_config = Config.parse_obj(get_driver().config.dict())
TEST_CHARACTER_NAME = "艾雅法拉"
JSON_PATH = Path(__file__).parent.parent / "_data" / "operator_info" / "json"
IMAGE_PATH = Path(__file__).parent.parent / "_data" / "operator_info" / "image"
FONT_PATH = Path(__file__).parent.parent / "_data" / "operator_info" / "font"
SAVE_PATH = Path(operator_config.operator_save_path)


class OperatorInfo:
    """小类，用来获取干员绘图必须的信息"""
    def __init__(self, name: str):
        self.operator_table = {}
        self.operator_result_info: dict = {}
        self.init_data: dict = {}

        self.init_info(name)

    def init_info(self, name: str):
        self._get_operator_basic_info(name)
        self._get_operator_all_skills_materials()
        self._get_operator_skills_materials()
        self._get_operator_evolve_materials()
        return self.operator_result_info

    def _get_operator_basic_info(self, name: str):
        """找到干员"""
        name = name or TEST_CHARACTER_NAME
        if not self.operator_table:
            with open(JSON_PATH / "character_table.json", "r", encoding="utf-8") as f:
                self.operator_table: dict = json.load(f)

        for code, data in self.operator_table.items():
            if data["name"] == name:
                self.operator_result_info["operator_code"] = code
                self.operator_result_info["rarity"] = data["rarity"]
                self.init_data = data

    def _get_operator_all_skills_materials(self):
        """获取干员技能1-7材料"""
        all_skills: list = self.init_data["allSkillLvlup"]  # 技能(1-7升级)
        all_skills_materials: dict = {}
        for all_skills_lvl, skill in enumerate(all_skills, start=1):
            cost: list = skill["lvlUpCost"]
            materials: dict = {material["id"]: material["count"] for material in cost}

            all_skills_materials[all_skills_lvl] = materials

        self.operator_result_info["all_skills_materials"] = all_skills_materials

    def _get_operator_skills_materials(self):
        """获取干员技能专精材料"""
        skills: list = self.init_data["skills"]  # 技能(专精升级)

        skills_materials: dict = {}  # 技能代码名 - 专精需要材料与数量
        for skill in skills:
            materials_list: list = []
            cost_condition: list = skill["levelUpCostCond"]
            if not cost_condition:
                continue
            for condition in cost_condition:
                cost: list = condition["levelUpCost"]
                materials: dict = {material["id"]: material["count"] for material in cost}
                materials_list.append(materials)
            skills_materials[skill["skillId"]] = materials_list
        self.operator_result_info["skills_materials"] = skills_materials

    def _get_operator_evolve_materials(self):
        """获取干员精英化材料"""
        phases: list = self.init_data["phases"]
        operator_rarity: int = self.init_data["rarity"]

        evolve_materials: dict = {}  # 精英化材料与数量
        evolve_lvl = 0
        for phase in phases:
            cost: list = phase["evolveCost"]
            if cost is None:
                continue
            materials: dict = {material["id"]: material["count"] for material in cost}

            evolve_lvl += 1
            evolve_materials[evolve_lvl] = materials

        if operator_rarity == 2:
            evolve_materials[1]["4001"] = 10000  # 龙门币

        elif operator_rarity == 3:
            evolve_materials[1]["4001"] = 15000  # 龙门币
            evolve_materials[2]["4001"] = 60000  # 龙门币
        elif operator_rarity == 4:
            evolve_materials[1]["4001"] = 20000  # 龙门币
            evolve_materials[2]["4001"] = 120000  # 龙门币
        elif operator_rarity == 5:
            evolve_materials[1]["4001"] = 30000  # 龙门币
            evolve_materials[2]["4001"] = 180000  # 龙门币
        self.operator_result_info["evolve_materials"] = evolve_materials

    @staticmethod
    def map_skill_code2name(skill_code: str) -> str:
        with open(JSON_PATH / "skill_table.json", "r", encoding="utf-8") as f:
            data: dict = json.load(f)
        return data[skill_code]["levels"][0]["name"]

    @property
    def code(self):
        return self.operator_result_info["operator_code"]

    @property
    def rarity(self):
        return self.operator_result_info["rarity"]

    @property
    def info(self):
        return self.operator_result_info

    @property
    def all_skills(self) -> dict:
        return self.info["all_skills_materials"]

    @property
    def evolve(self) -> dict:
        return self.info["evolve_materials"]

    @property
    def skills(self) -> dict:
        return self.info["skills_materials"]


class BuildOperatorImage:
    """绘图"""

    def __init__(self, operator: OperatorInfo, font_file_en: Union[str, Path] = str(FONT_PATH / "Arknights-en.ttf"), font_file_zh: Union[str, Path] = str(FONT_PATH / "Arknights-zh.otf")):
        self.operator = operator  # 干员
        self.font_file_en = font_file_en  # 字体文件
        self.font_file_zh = font_file_zh  # 字体文件

        self.is_all_skills: bool = operator.rarity > 1  # 是否有技能升级(1, 2星无)
        self.is_skills: bool = operator.rarity > 2  # 是否有技能专精(1, 2, 3星无)
        self.is_evolve_1: bool = self.is_all_skills  # 能否精一(1, 2星无)
        self.is_evolve_2: bool = self.is_skills  # 能否精二2(1, 2, 3星无)

        self.background: Image  # 背景
        self.avatar: Image  # 干员头图

        self.all_skills: Image  # 1-7部分
        self.skills: Image  # 技能专精部分
        self.evolve: Image  # 精英化部分

        self.result: Image  # 整合部分

    def build_whole_image(self) -> Path:
        None if os.path.exists(SAVE_PATH) else os.mkdir(SAVE_PATH)
        file = SAVE_PATH / f"{self.operator.code}.png"

        all_skills_img = self._build_all_skills()  # 1-7
        skills_img = self._build_skills()  # 专精
        evolve_img = self._build_evolve()  # 精英化
        skin_img = self._build_skin()  # 立绘

        main_background = Image.new(mode="RGBA", size=(1632, 744), color=(100, 100, 100, 200))
        Draw(main_background).rectangle(xy=(0, 0, 1632, 744), outline=(10, 10, 10), width=4)  # 最外围边框

        if self.operator.rarity < 2:  # 只有精一立绘
            main_background.paste(im=skin_img, box=(-140, 0), mask=skin_img.split()[3])
        else:
            main_background.paste(im=skin_img, box=(-140, -140), mask=skin_img.split()[3])
        main_background.paste(im=all_skills_img, box=(528, 48), mask=all_skills_img.split()[3])
        main_background.paste(im=skills_img, box=(528, 288), mask=skills_img.split()[3])
        main_background.paste(im=evolve_img, box=(48, 480), mask=evolve_img.split()[3])

        with contextlib.suppress(Exception):
            main_background.save(file)
        return file

    def _build_skin(self) -> Image:
        """立绘，优先用精二"""
        return Image.open(IMAGE_PATH / "skin" / self.get_operator_skin()).convert(mode="RGBA")

    def _build_all_skills(self) -> Image:
        """绘制 1-7 部分"""
        font_en = ImageFont.truetype(self.font_file_en, 24)
        main_background = Image.new(mode="RGBA", size=(1056, 216), color=(235, 235, 235, 100))  # 底图
        if not self.is_all_skills:  # 没有技能升级，如一星二星
            font_zh = ImageFont.truetype(self.font_file_zh, 48)
            Draw(main_background).text(xy=(528, 132), anchor="ms", align="center", text="该干员无技能升级", font=font_zh, fill=(255, 255, 255, 255))
            return main_background

        img_head_shadow = Image.new(mode="RGBA", size=(1056, 24), color=(175, 175, 175, 160))  # 顶部阴影
        main_background.paste(im=img_head_shadow, box=(0, 0), mask=img_head_shadow.split()[3])

        all_skills = self.operator.all_skills
        backgrounds = []
        for lvl, skill in all_skills.items():
            background = Image.new(mode="RGBA", size=(352, 96), color=(235, 235, 235, 100))  # 底图
            text_shadow = Image.new(mode="RGBA", size=(96, 96), color=(205, 205, 205, 130))  # 左侧阴影
            background.paste(im=text_shadow, box=(0, 0), mask=text_shadow.split()[3])
            self.text_border(text=f"{lvl}~{lvl + 1}", draw=Draw(background), x=48, y=60, font=font_en, shadow_colour=(0, 0, 0, 255), fill_colour=(255, 255, 255, 255))  # 顶部文字
            # Draw(background).text(xy=(48, 60), anchor="ms", align="center", text=f"{lvl}~{lvl + 1}", font=font_en, fill=(255, 255, 255, 255))  # 左侧文字

            item_count = 0
            for mat, count in skill.items():
                icon = self.get_material_icon(material_code=mat).resize(size=(64, 64))  # 材料图标大小
                self.text_border(text=str(count), draw=Draw(icon), x=45, y=57, font=font_en, shadow_colour=(255, 255, 255, 255), fill_colour=(0, 0, 0, 255))  # 右下角的数字
                background.paste(im=icon, box=(112 + item_count, 16), mask=icon.split()[3])  # 透明度粘贴(r,g,b,a)
                item_count += 80
            backgrounds.append(background)

        # 粘到大图上
        for idx, bg in enumerate(backgrounds):
            if idx < 3:
                main_background.paste(im=bg, box=(idx * 352, 24))
            else:
                main_background.paste(im=bg, box=((idx - 3) * 352, 120))
        font_zh = ImageFont.truetype(self.font_file_zh, 16)
        # Draw(main_background).text(xy=(528, 20), anchor="ms", align="center", text="技 能 升 级", font=font_zh, fill=(255, 255, 255, 255))  # 最顶部的字
        self.text_border(text="技 能 升 级", draw=Draw(main_background), x=528, y=20, font=font_zh, shadow_colour=(0, 0, 0, 255), fill_colour=(255, 255, 255, 255))  # 顶部文字

        main_draw = Draw(main_background)
        # main_draw.rectangle(xy=(0, 0, 1056, 216), outline=(50, 50, 50), width=4)  # 最外围边框
        main_draw.line(xy=(0, 0, 1056, 0), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(0, 0, 0, 216), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(1054, 0, 1054, 216), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(0, 214, 1056, 214), width=4, fill=(50, 50, 50))

        main_draw.line(xy=(0, 24, 1056, 24), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(0, 120, 1056, 120), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(96, 24, 96, 216), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(352, 24, 352, 216), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(448, 24, 448, 216), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(704, 24, 704, 216), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(800, 24, 800, 216), width=2, fill=(50, 50, 50))
        return main_background

    def _build_skills(self) -> Image:
        """绘制技能专精部分"""
        font_en = ImageFont.truetype(self.font_file_en, 24)
        font_zh = ImageFont.truetype(self.font_file_zh, 24)
        main_background = Image.new(mode="RGBA", size=(1056, 408), color=(235, 235, 235, 100))  # 底图
        if not self.is_skills:  # 没有技能升专精，如一星二星三星
            font_zh = ImageFont.truetype(self.font_file_zh, 48)
            Draw(main_background).text(xy=(528, 216), anchor="ms", align="center", text="该干员无技能专精", font=font_zh, fill=(255, 255, 255, 255))
            return main_background

        img_head_shadow = Image.new(mode="RGBA", size=(1056, 24), color=(175, 175, 175, 160))  # 顶部阴影
        main_background.paste(im=img_head_shadow, box=(0, 0), mask=img_head_shadow.split()[3])

        skills = self.operator.skills
        main_backgrounds = []
        for skill, data in skills.items():  # 对每个技能
            skill_main_backgrounds = Image.new(mode="RGBA", size=(352, 384), color=(235, 235, 235, 100))  # 每个技能的底图
            icon_shadow = Image.new(mode="RGBA", size=(96, 96), color=(205, 205, 205, 130))  # 左侧阴影
            skill_main_backgrounds.paste(im=icon_shadow, box=(0, 0), mask=icon_shadow.split()[3])
            skill_icon = self.get_skill_icon(skill).resize(size=(96, 96))  # 技能图标
            skill_main_backgrounds.paste(im=skill_icon, box=(0, 0))
            skill_name = self.operator.map_skill_code2name(skill)  # 技能名
            draw = Draw(skill_main_backgrounds)
            self.text_border(text=skill_name, draw=draw, x=224, y=60, font=font_zh, shadow_colour=(0, 0, 0, 255), fill_colour=(255, 255, 255, 255))

            backgrounds = []
            for idx, level in enumerate(data):  # 对每层专精
                background = Image.new(mode="RGBA", size=(352, 96), color=(235, 235, 235, 100))  # 小底图
                text_shadow = Image.new(mode="RGBA", size=(96, 96), color=(205, 205, 205, 130))  # 左侧阴影
                background.paste(im=text_shadow, box=(0, 0), mask=text_shadow.split()[3])

                level_icon = Image.open(IMAGE_PATH / "skill" / f"skill_lvl{idx + 1}.png", mode="r").convert("RGBA").resize(size=(96, 96))  # 专精图标
                background.paste(im=level_icon, box=(0, 0), mask=level_icon.split()[3])

                item_count = 0
                for mat, count in level.items():
                    icon = self.get_material_icon(mat).resize(size=(64, 64))  # 材料图标大小
                    self.text_border(text=str(count), draw=Draw(icon), x=45, y=57, font=font_en, shadow_colour=(255, 255, 255, 255), fill_colour=(0, 0, 0, 255))
                    background.paste(im=icon, box=(112 + item_count, 16), mask=icon.split()[3])
                    item_count += 80
                backgrounds.append(background)

            # 粘到大图上
            for idx, bg in enumerate(backgrounds):
                skill_main_backgrounds.paste(im=bg, box=(0, (idx + 1) * 96))
            main_backgrounds.append(skill_main_backgrounds)

        for idx, bg in enumerate(main_backgrounds):
            main_background.paste(im=bg, box=(idx * 352, 24))
        font_zh = ImageFont.truetype(self.font_file_zh, 16)
        # Draw(main_background).text(xy=(528, 20), anchor="ms", align="center", text="技 能 专 精", font=font_zh, fill=(255, 255, 255, 255))  # 最顶部的字
        self.text_border(text="技 能 专 精", draw=Draw(main_background), x=528, y=20, font=font_zh, shadow_colour=(0, 0, 0, 255), fill_colour=(255, 255, 255, 255))  # 左侧文字

        main_draw =Draw(main_background)
        main_draw.line(xy=(0, 0, 1056, 0), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(0, 0, 0, 408), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(1054, 0, 1054, 408), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(0, 406, 1056, 406), width=4, fill=(50, 50, 50))

        main_draw.line(xy=(0, 24, 1056, 24), width=2, fill=(50, 50, 50))

        main_draw.line(xy=(0, 120, 1056, 120), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(0, 216, 1056, 216), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(0, 312, 1056, 312), width=2, fill=(50, 50, 50))

        main_draw.line(xy=(96, 24, 96, 408), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(352, 24, 352, 408), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(448, 24, 448, 408), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(704, 24, 704, 408), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(800, 24, 800, 408), width=2, fill=(50, 50, 50))
        return main_background

    def _build_evolve(self) -> Image:
        """绘制精英化部分"""
        font_en = ImageFont.truetype(self.font_file_en, 24)
        font_zh = ImageFont.truetype(self.font_file_zh, 24)
        main_background = Image.new(mode="RGBA", size=(432, 216), color=(235, 235, 235, 100))  # 底图
        if not self.is_evolve_1:  # 没有技能升专精，如一星二星三星
            font_zh = ImageFont.truetype(self.font_file_zh, 48)
            Draw(main_background).text(xy=(216, 132), anchor="ms", align="center", text="该干员无法精英化", font=font_zh, fill=(255, 255, 255, 255))
            return main_background

        img_head_shadow = Image.new(mode="RGBA", size=(432, 24), color=(175, 175, 175, 160))  # 顶部阴影
        main_background.paste(im=img_head_shadow, box=(0, 0), mask=img_head_shadow.split()[3])

        evolve = self.operator.evolve
        backgrounds = []
        for lvl, data in evolve.items():
            background = Image.new(mode="RGBA", size=(432, 96), color=(235, 235, 235, 100))  # 底图
            text_shadow = Image.new(mode="RGBA", size=(96, 96), color=(205, 205, 205, 130))  # 左侧阴影
            background.paste(im=text_shadow, box=(0, 0), mask=text_shadow.split()[3])
            draw = Draw(background)
            self.text_border(text=f"精 {lvl}", draw=draw, x=48, y=60, font=font_zh, shadow_colour=(0, 0, 0, 255), fill_colour=(255, 255, 255, 255))

            item_count = 0
            for mat, count in data.items():
                icon = self.get_material_icon(material_code=mat).resize(size=(64, 64))  # 材料图标大小
                if count >= 10000:
                    count = f"{count / 10000:.0f}w"
                    font = ImageFont.truetype(self.font_file_en, 14)
                    self.text_border(text=str(count), draw=Draw(icon), x=45, y=52, font=font, shadow_colour=(255, 255, 255, 255), fill_colour=(0, 0, 0, 255))  # 右下角的数字
                else:
                    self.text_border(text=str(count), draw=Draw(icon), x=45, y=57, font=font_en, shadow_colour=(255, 255, 255, 255), fill_colour=(0, 0, 0, 255))  # 右下角的数字
                background.paste(im=icon, box=(112 + item_count, 16), mask=icon.split()[3])  # 透明度粘贴(r,g,b,a)
                item_count += 80
            backgrounds.append(background)

        # 粘到大图上
        for idx, bg in enumerate(backgrounds):
            main_background.paste(im=bg, box=(0, 24 + idx * 96))
        font_zh = ImageFont.truetype(self.font_file_zh, 16)
        # Draw(main_background).text(xy=(216, 20), anchor="ms", align="center", text="精英化", font=font_zh, fill=(255, 255, 255, 255))  # 最顶部的字
        self.text_border(text="精 英 化", draw=Draw(main_background), x=216, y=20, font=font_zh, shadow_colour=(0, 0, 0, 255), fill_colour=(255, 255, 255, 255))  # 左侧文字

        main_draw =Draw(main_background)
        main_draw.line(xy=(0, 0, 432, 0), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(0, 0, 0, 216), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(430, 0, 430, 216), width=4, fill=(50, 50, 50))
        main_draw.line(xy=(0, 214, 432, 214), width=4, fill=(50, 50, 50))

        main_draw.line(xy=(0, 24, 432, 24), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(0, 120, 432, 120), width=2, fill=(50, 50, 50))
        main_draw.line(xy=(96, 24, 96, 216), width=2, fill=(50, 50, 50))

        return main_background

    def get_operator_avatar(self) -> Image:
        return Image.open(IMAGE_PATH / "avatar" / f"{self.operator.code}.png").convert(mode="RGBA")

    def get_operator_skin(self) -> Union[str, bytes]:
        skins = os.listdir(IMAGE_PATH / "skin")
        result = [name for name in skins if self.operator.code in name]
        result.sort()
        try:
            return result[1]
        except IndexError:
            return result[0]

    @classmethod
    def _map_skill_icon_id(cls, skill_code: str) -> Image:
        """技能也有不同的"""
        with open(JSON_PATH / "skill_table.json", "r", encoding="utf-8") as f:
            skill_table = json.load(f)
        return next((data["iconId"] or data["skillId"] for skill, data in skill_table.items() if skill == skill_code), None)

    @classmethod
    def get_skill_icon(cls, skill_code: str) -> Image:
        # print(skill_code)
        return Image.open(IMAGE_PATH / "skill" / f"skill_icon_{cls._map_skill_icon_id(skill_code)}.png").convert( mode="RGBA")

    @classmethod
    def _map_item_icon_id(cls, item_code: str) -> str:
        """物品的代码名和图标名不同"""
        with open(JSON_PATH / "item_table.json", "r", encoding="utf-8") as f:
            item_table = json.load(f)
        return next((data["iconId"] for item, data in item_table["items"].items() if item == item_code), None)

    @classmethod
    def get_material_icon(cls, material_code: str) -> Image:
        return Image.open(IMAGE_PATH / "item" / f"{cls._map_item_icon_id(material_code)}.png").convert(mode="RGBA")

    @staticmethod
    def text_border(text: str, draw: ImageDraw, x: int, y: int, font: ImageFont, shadow_colour: tuple, fill_colour: tuple):
        """文字加边框"""
        draw.text((x - 1, y), text=text, anchor="ms", font=font, fill=shadow_colour)
        draw.text((x + 1, y), text=text, anchor="ms", font=font, fill=shadow_colour)
        draw.text((x, y - 1), text=text, anchor="ms", font=font, fill=shadow_colour)
        draw.text((x, y + 1), text=text, anchor="ms", font=font, fill=shadow_colour)

        draw.text((x - 1, y - 1), text=text, anchor="ms", font=font, fill=shadow_colour)
        draw.text((x + 1, y - 1), text=text, anchor="ms", font=font, fill=shadow_colour)
        draw.text((x - 1, y + 1), text=text, anchor="ms", font=font, fill=shadow_colour)
        draw.text((x + 1, y + 1), text=text, anchor="ms", font=font, fill=shadow_colour)

        draw.text((x, y), text=text, anchor="ms", font=font, fill=fill_colour)


__all__ = [
    "OperatorInfo",
    "BuildOperatorImage"
]



from pydantic import BaseModel, Extra


class Config(BaseModel, extra=Extra.ignore):
    """新活动数据/图片相关配置"""
    activities_img_path: str = "data/arktools/activity/img"
    activities_data_path: str = "data/arktools/activity/data"



"""泰拉记事社, https://terra-historicus.hypergryph.com/"""
import time

from nonebot.log import logger

from ..._exceptions import APICodeException
from ..._utils import get_api, request_

APIS = get_api("terra-historicus")


async def get_comic_news():
    """获取泰拉记事社的最新漫画消息"""
    url = APIS["recentUpdate"]["url"]
    method = APIS["recentUpdate"]["method"]
    response = await request_(url=url, method=method)

    data = response.json()
    if data['code'] != 0:
        logger.error("泰拉记事社最新漫画获取失败")
        raise APICodeException(status=response['code'])

    data = response.json()['data']
    logger.info(f"comic_data: {data}")
    result = []
    for _ in data[:3]:
        result.append(await process_comic_news(_))
    logger.info(f"comic_result: {result}")
    return result


async def process_comic_news(news_data: dict) -> dict:
    """获取公告中的图片等信息"""
    cid = news_data['comicCid']
    url = APIS["comic-data"]["url"].format(cid)
    method = APIS["comic-data"]["method"]
    response = await request_(url=url, method=method)

    data = response.json()
    if data['code'] != 0:
        raise APICodeException(status=response['code'])

    data = response.json()['data']

    title = data['title']
    subtitle = data['subtitle']
    authors = data['authors']
    keywords = data['keywords']
    cover = data['cover']

    date = data['updateTime']
    date = time.localtime(date)
    date = time.strftime("%Y-%m-%d", date)

    return {
        "title": title,
        "subtitle": subtitle,
        "authors": authors,
        "keywords": keywords,
        "date": date,
        "cover": cover,
        "url": f"https://terra-historicus.hypergryph.com/comic/{cid}"
    }





"""方舟官网, https://ak.hypergryph.com/"""
from lxml import etree

from ..._utils import request_


async def pre_process():
    url = "https://ak.hypergryph.com/news/"
    response = await request_(url=url)
    text = response.text
    dom = etree.HTML(text, etree.HTMLParser())

    result = await get_arknights_activities(dom)

    result = await get_arknights_news(dom)

    ...


async def get_arknights_activities(dom):
    """活动(卡池、活动、合约)"""
    activities = dom.xpath("//ol[@data-category-key='ACTIVITY']/li/a/@href")[0]
    time = dom.xpath("//ol[@data-category-key='ACTIVITY']/li/a/span[@class='articleItemDate']/text()")[0]
    url = f"https://ak.hypergryph.com/{activities}"
    response = await request_(url=url)
    text = response.text
    dom_ = etree.HTML(text, etree.HTMLParser())
    paragraph = dom_.xpath("//p/strong/text() | //p/text[contains(string(), '时间')]")

    ...


async def get_arknights_news(dom):
    """新闻(制作组通讯)"""
    news = dom.xpath("//ol[@data-category-key='NEWS']/li/a/@href")[0]
    time = dom.xpath("//ol[@data-category-key='NEWS']/li/a/span[@class='articleItemDate']/text()")[0]
    url = f"https://ak.hypergryph.com/{news}"
    response = await request_(url=url)
    text = response.text
    dom_ = etree.HTML(text, etree.HTMLParser())

    ...







"""剿灭、蚀刻章、合约等活动到期提醒"""
import os

import nonebot
from nonebot import on_regex
from nonebot.adapters.onebot.v11 import Message
from nonebot.log import logger
from nonebot_plugin_apscheduler import scheduler

from .config import Config
from .data_source import get_activities

global_config = nonebot.get_driver().config
activity_config = Config(**global_config.dict())

latest_activity = on_regex(r'[查看询]*方舟[最]*新+[活动闻]*', priority=5, block=True)


@latest_activity.handle()
async def _():
    rst_msg = await get_activities(is_force=True)
    if isinstance(rst_msg, Message):
        rst = rst_msg
    elif isinstance(rst_msg, str):
        rst = (
            f"方舟最新活动截图失败！\n请更换至非windows平台部署本插件\n或检查网络连接并稍后重试"
            f"最新的活动信息请见链接: {rst_msg}"
        )
    else:
        rst = "无法获得方舟最新活动信息！请稍后重试"
    await latest_activity.finish(rst)


@scheduler.scheduled_job(
    "cron",
    hour=4,
    minute=1,
)
async def _():
    try:
        await get_activities(is_force=True, is_cover=True)
    except Exception as e:
        logger.error(f"方舟最新活动检查失败！{type(e)}: {e}")


driver = nonebot.get_driver()
@driver.on_startup
async def _():
    if not os.path.exists(activity_config.activities_data_path):
        os.makedirs(activity_config.activities_data_path)
    if not os.path.exists(activity_config.activities_img_path):
        os.makedirs(activity_config.activities_img_path)
    await get_activities(is_force=True)



from .._utils import request_
from typing import List


TAGS = [
    "近卫干员", "狙击干员", "重装干员", "医疗干员", "辅助干员", "术士干员", "特种干员", "先锋干员",
    "近战位", "远程位", "高级资深干员", "资深干员", "支援机械", "男性干员", "女性干员",
    "控场", "爆发", "治疗", "支援", "新手", "费用回复", "输出", "生存", "群攻", "防护", "减速",
    "削弱", "快速复活", "位移", "召唤"
]


async def get_recommend_tags(taglist: List[str]):
    taglist = await preprocess_tags(taglist)

    url = "http://110.40.221.138:8000/api/ark/tools/recruit/recommend"
    headers = {"accept": "application/json"}
    data = {"tags": taglist}
    response = await request_(url=url, headers=headers, data=data, method='POST')

    if response.status_code != 200:
        return None

    recommend_datas = response.json()['recruitRecommends']
    result = ""
    for data in recommend_datas:
        tags_info = data['recommendTags']
        operator_info = data['recommendOperatorInfos']
        operators = {}
        for op in operator_info:
            if op['rarity'] in operators:
                operators[op['rarity']].append(op['name'])

            else:
                operators[op['rarity']] = [op['name']]
        result += f"\n推荐tag：{'+'.join(tags_info)}\n"
        for rarity in range(1, 7):
            if rarity in operators:
                result += f"{'★' * rarity}: {', '.join(operators[rarity])}\n"

    return result


async def preprocess_tags(taglist: List[str]) -> List[str]:
    for idx, tag in enumerate(taglist):
        if "术师" in tag:
            taglist[idx] = tag.replace("术师", "术士")

        if tag in {'近卫', '重装', '先锋', '医疗', '辅助', '特种', '术士', '狙击'}:
            taglist[idx] = f"{tag}干员"
        if tag in {"高资", "高姿", "高级"}:
            taglist[idx] = "高级资深干员"
        if tag in {"资深", "资干"}:
            taglist[idx] = "资深干员"
        if tag in {"快活", "复活"}:
            taglist[idx] = "快速复活"
        if tag in {"近战", "地面"}:
            taglist[idx] = "近战位"
        if tag in {"远程", "高台"}:
            taglist[idx] = "远程位"
        if tag in {"费回", "回费", "费用恢复", "费恢"}:
            taglist[idx] = "费用回复"
        if tag in {"机械", "小车"}:
            taglist[idx] = "支援机械"
        if tag in {"男性", "男", "男人"}:
            taglist[idx] = "男性干员"
        if tag in {"女性", "女", "女人"}:
            taglist[idx] = "女性干员"

    taglist = [_ for _ in taglist if _ in TAGS]
    if len(taglist) > 5:
        taglist = taglist[:5]

    return taglist



"""API 相关异常"""
from . import ArkBaseException


class APINonExistenceException(ArkBaseException):
    """API不存在"""
    def __init__(self, msg: str = "该API不存在", **kwargs):
        super().__init__(msg, **kwargs)
        self.msg = msg


class APICodeException(ArkBaseException):
    """接口响应码异常"""
    def __init__(self, status: int = 200, msg: str = "接口响应码错误", **kwargs):
        super().__init__(msg, **kwargs)
        self.status = status

    def __repr__(self):
        return f"{self.status}: {self.msg}"



import contextlib
from typing import Optional

import nonebot
from nonebot import Driver
from nonebot.log import logger
from playwright.async_api import Browser, async_playwright

driver: Driver = nonebot.get_driver()


_browser: Optional[Browser] = None


async def init(**kwargs) -> Optional[Browser]:
    global _browser
    try:
        browser = await async_playwright().start()
        _browser = await browser.chromium.launch(**kwargs)
        return _browser
    except NotImplementedError:
        logger.warning("初始化 playwright 失败，请依次进行如下操作：")
        logger.warning("1.请检查 playwright 库和 uvicorn 库是否为最新版本")
        logger.warning("2.请删除 env 文件中的 “FASTAPI_RELOAD=true” 语句")
        logger.warning("3.请不要在 windows 环境下部署")
    except Exception as e:
        logger.warning(f"启动chromium发生错误 {type(e)}：{e}")
        if _browser:
            await _browser.close()
    return None


async def get_browser(**kwargs) -> Browser:
    return _browser or await init(**kwargs)


# @driver.on_startup
def install():
    """自动安装、更新 Chromium"""
    logger.info("正在检查 Chromium 更新")
    import sys
    from playwright.__main__ import main

    sys.argv = ["", "install", "chromium"]
    with contextlib.suppress(SystemExit):
        main()



import httpx
from nonebot import logger
from .._exceptions import *


async def request_(url, method: str = "GET", *, retry: int = 5, **kwargs):
    if method in {"get", "GET"}:
        return await async_get(url, retry=retry, **kwargs)
    elif method in {"post", "POST"}:
        return await async_post(url, retry=retry, **kwargs)
    else:
        raise NoMethodException


async def async_get(url, *, retry: int = 5, **kwargs):
    async with httpx.AsyncClient() as client:
        for times in range(retry):
            try:
                return await client.get(url, **kwargs)
            except httpx.TimeoutException:
                logger.warning(f"get请求第{times+1}次失败...")
                continue
            except Exception:
                raise
        raise


async def async_post(url, *, retry: int = 5, **kwargs):
    async with httpx.AsyncClient() as client:
        for times in range(retry):
            try:
                return await client.post(url, **kwargs)
            except httpx.TimeoutException:
                logger.warning(f"post请求第{times+1}次失败...")
                continue
            except Exception:
                raise
        raise
